---
id: config
title: Configuration file
---

To configure Remotion, create a `remotion.config.ts` file in the root of your Remotion project.

These options will apply to CLI commands such as `npm start` and `npm run build`, but have no effect in the Node.JS API.

You can control several behaviors of Remotion here.

```tsx
import {Config} from 'remotion';

Config.Rendering.Concurrency(8);
Config.Output.setPixelFormat('yuv444p');
Config.Output.setCodec('h265');
```

## Bundling

### overrideWebpackConfig()

_Available from Version 1.1._

Allows you to insert your custom Webpack config. [See the page about custom Webpack configs](/docs/webpack) for more information.

```tsx
Config.Bundling.overrideWebpackConfig((currentConfiguration) => {
  // Return a new Webpack configuration
});
```

### setCachingEnabled()

_Available from Version 2.0._

Enable or disable webpack caching. Default is `true` which will make the Webpack step in the first run a bit slower but will massively speed up subsequent runs. We recommend to keep this option enabled in all cases and encourage to report issues on Github if you encounter some.

```tsx
Config.Bundling.setCachingEnabled(false);
```

The [command line flag](/docs/cli) `--bundle-cache` will take precedence over this option.

## Log

### setLevel()

_Available from Version 2.0.1_

Increase or decrease the amount of log messages in the CLI.
Acceptable values:

- `error`: Silent except error messages.
- `warning`: Only showing errors and warnings.
- `info` (_default_): Default output - besides errors and warnings, prints progress and output location.
- `verbose`: All of the above, plus browser logs and other debug info.

```tsx
Config.Log.setLevel('verbose');
```

The [command line flag](/docs/cli) `--log` will take precedence over this option.

## Puppeteer

### setBrowserExecutable()

_Available from Version 1.5._

Set a custom Chrome or Chromium executable path. By default Remotion will try to find an existing version of Chrome on your system and if not found, it will download one. This flag is useful if you don't have Chrome installed in a standard location and you want to prevent downloading an additional browser or need [support for the H264 codec](/docs/video#codec-support).

```ts
Config.Puppeteer.setBrowserExecutable('/usr/bin/google-chrome-stable')
```

The [command line flag](/docs/cli) `--browser-executable` will take precedence over this option.

## Rendering

### setConcurrency()

Sets how many Puppeteer instances will work on rendering your video in parallel.
Default: `null`, meaning **half of the threads** available on your CPU.

```ts
Config.Rendering.setConcurrency(8)
```

The [command line flag](/docs/cli) `--concurrency` will take precedence over this option.

:::tip
Try to set your concurrency to `os.cpus().length` to all the threads available on your CPU for faster rendering. The drawback is that other parts of your system might slow down.
:::

### setImageFormat()

_Available from Version 1.4._

Determines which in which image format to render the frames. Either:

- `jpeg` - the fastest option (default from v1.1)
- `png` - slower, but supports transparency
- `none` - don't render images, just calculate audio information (available from v2.0)

```tsx
Config.Rendering.setImageFormat('png')
```

The [command line flag](/docs/cli) `--image-format` will take precedence over this option.

### setFrameRange()

_Available from Version 2.0._

Pass a number to render a still frame or a tuple to render a subset of a video. The frame sequence is zero-indexed.

```tsx
Config.Rendering.setFrameRange(90); // To render only the 91st frame
```

or

```tsx
Config.Rendering.setFrameRange([0, 20]); // Render a video only containing the first 21 frames
```

### setQuality()

The JPEG quality of each frame. Must be a number between 0 and 100. Will not work if you render PNG frames. [Default: 80](https://github.com/chromium/chromium/blob/99314be8152e688bafbbf9a615536bdbb289ea87/headless/lib/browser/protocol/headless_handler.cc#L32).

```tsx
Config.Rendering.setQuality(90);
```

## Output

### setOverwriteOutput()

Set this to `false` to prevent overwriting Remotion outputs when they already exists.

```tsx
Config.Output.setOverwriteOutput(false)
```

The [command line flag](/docs/cli) `--overwrite` will take precedence over this option.

:::info
In version 1.x, the default behavior was inverse - Remotion would not override by default.
:::

### setPixelFormat()

Controls the pixel format in FFMPEG. [Read more about it here.](https://trac.ffmpeg.org/wiki/Chroma%20Subsampling0) Acceptable values: `yuv420p`, `yuv422p`, `yuv444p`, `yuv420p10le`, `yuv422p10le`, `yuv444p10le`. Since v1.4, `yuva420p` is also supported for transparent WebM videos.
Default value: `yuv420p`

```tsx
Config.Output.setPixelFormat('yuv420p')
```

The [command line flag](/docs/cli) `--pixel-format` will take precedence over this option.

### setCodec()

_Available from Version 1.4._

Choose one of the supported codecs: `h264` _(default)_, `h265`, `vp8`, `vp9`.

- `h264` is the classic MP4 file as you know it.
- `h265` is the successor of H264, with smaller file sizes. Also known as HEVC. Poor browser compatibility.
- `vp8` is the codec for WebM.
- `vp9` is the next-generation codec for WebM. Lower file size, longer compression time.
- `mp3` will export audio only as an MP3 file.
- `wav` will export audio only as an WAV file.
- `aac` will export audio only as an AAC file.

```tsx
Config.Output.setCodec('h265');
```

**See also**: [Encoding guide](/docs/encoding)

### setImageSequence()

_Available from Version 1.4._

Set to true if you want to output an image sequence instead of a video.

```tsx
Config.Output.setImageSequence(true);
```

### ~~setOutputFormat()~~

_Deprecated_. Use `setCodec()` and `setImageSequence()` instead.

Either `'mp4'` or `'png-sequence'`.

```tsx
Config.Output.setOutputFormat('mp4');
```

The [command line flags](/docs/cli) `--sequence` and `--codec` will take precedence over this option.

The [command line flag](/docs/cli) `--quality` will take precedence over this option.

### setCrf()

_Available from Version 1.4._

The "Constant Rate Factor" (CRF) of the output. [Use this setting to tell FFMPEG how to trade off size and quality.](encoding#controlling-quality-using-the-crf-setting)

Ranges for CRF scale, by codec:

- `h264` crf range is 0-51 where crf 18 is _default_.
- `h265` crf range is 0-51 where crf 23 is _default_.
- `vp8` crf range is 4-63 where crf 9 is _default_.
- `vp9` crf range is 0-63 where crf 28 is _default_.

The lowest value is lossless, and the highest value is the worst quality possible. Higher values decrease the filesize at the cost of quality.

The range is exponential, so increasing the CRF value +6 results in roughly half the bitrate / file size, while -6 leads to roughly twice the bitrate.

Choose the highest CRF value that still provides an acceptable quality. If the output looks good, then try a higher value. If it looks bad, choose a lower value.

```tsx
Config.Output.setCrf(16);
```

The [command line flag](/docs/cli) `--crf` will take precedence over this option.

## See also

- [Encoding guide](/docs/encoding)
